@@ -1,6 +1,6 @@
 ##
 ##
-## Copyright (c) 2009, Brandon Whitcher and Volker Schmid
+## Copyright (c) 2009,2010 Brandon Whitcher and Volker Schmid
 ## All rights reserved.
 ## 
 ## Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
 ## 
 ## $Id: dcemri_bayes.R 332 2010-01-29 16:54:07Z bjw34032 $
 ##
+
 #############################################################################
 ## setGeneric("dcemri.bayes")
 #############################################################################
@@ -40,56 +41,59 @@ setGeneric("dcemri.bayes",
 setMethod("dcemri.bayes", signature(conc="array"), 
           function(conc, time, img.mask, model="extended",
                          aif=NULL, user=NULL, nriters=3500, thin=3,
-                         burnin=500, tune=267, tau.ktrans=1,
-                         tau.kep=tau.ktrans, ab.vp=c(1,19),
+                         burnin=500, tune=267, ab.ktrans=c(0,1),
+                         ab.kep=ab.ktrans, ab.vp=c(1,19),
                          ab.tauepsilon=c(1,1/1000), samples=FALSE,
-                         multicore=FALSE, verbose=FALSE, ...) .dcemriWrapper("dcemri.bayes", conc, time, img.mask, model,
+                         multicore=FALSE, verbose=FALSE, dic=FALSE, ...)
+          .dcemriWrapper("dcemri.bayes", conc, time, img.mask, model,
                          aif, user, nriters, thin,
-                         burnin, tune, tau.ktrans,
-                         tau.kep, ab.vp,
+                         burnin, tune, ab.ktrans,
+                         ab.kep, ab.vp,
                          ab.tauepsilon, samples,
-                         multicore, verbose, ...))
-
+                         multicore, verbose, dic, ...))
 
 .dcemri.bayes.single <- function(conc, time, nriters=3500, thin=3,
-                                burnin=500, tune=267, tau.gamma=1,
-                                tau.theta=1, ab.vp=c(1,19),
+                                burnin=500, tune=267, ab.gamma=c(0,1),
+                                ab.theta=c(0,1), ab.vp=c(1,19),
                                 ab.tauepsilon=c(1,1/1000), aif.model=0,
                                 aif.parameter=c(2.4,0.62,3,0.016), vp=1) {
 
-  if (sum(is.na(conc)) > 0)
+  if (sum(is.na(conc)) > 0) {
     return(NA)
-  else {
+  } else {
     n <- floor((nriters - burnin) / thin)
-    if (tune > (0.5*nriters))
+    if (tune > nriters/2) {
       tune <- floor(nriters/2)
+    }
     singlerun <- .C("dce_bayes_run_single",
                     as.integer(c(nriters, thin, burnin, tune)),
                     as.double(conc),
-                    as.double(tau.gamma),
-                    as.double(tau.theta),
+                    as.double(ab.gamma),
+                    as.double(ab.theta),
                     as.double(ab.vp),
                     as.double(ab.tauepsilon),
                     as.double(c(aif.model, aif.parameter)),
-                    as.integer(vp),
+                    as.integer(vp), # is this correct?
                     as.double(time),
                     as.integer(length(time)),
                     as.double(rep(0,n)),
                     as.double(rep(0,n)),
                     as.double(rep(0,n)),
                     as.double(rep(0,n)), 
+                    as.double(rep(0,n)), 
                     PACKAGE="dcemriS4")    
     return(list("ktrans"= singlerun[[11]], "kep"= singlerun[[12]],
-                "vp"= singlerun[[13]], "sigma2"= 1/singlerun[[14]]))
+                "vp"= singlerun[[13]], "sigma2"= 1/singlerun[[14]],
+                "deviance" = singlerun[[15]]))
   }
 }
 
 .dcemri.bayes <- function(conc, time, img.mask, model="extended",
                          aif=NULL, user=NULL, nriters=3500, thin=3,
-                         burnin=500, tune=267, tau.ktrans=1,
-                         tau.kep=tau.ktrans, ab.vp=c(1,19),
+                         burnin=500, tune=267, ab.ktrans=c(0,1),
+                         ab.kep=ab.ktrans, ab.vp=c(1,19),
                          ab.tauepsilon=c(1,1/1000), samples=FALSE,
-                         multicore=FALSE, verbose=FALSE, ...) {
+                         multicore=FALSE, verbose=FALSE, dic=FALSE...) {
 
   ## dcemri.bayes - a function for fitting 1-compartment PK models to
   ## DCE-MRI images using Bayes inference
@@ -107,30 +111,58 @@ setMethod("dcemri.bayes", signature(conc="array"),
   ##         (ktranserror and keperror), samples if samples=TRUE
   ##
 
-  extract.samples <- function(sample, I, J, K, NRI) {
+  extract.samples <- function(sample, mask, I, J, K, NRI) {
     A <- array(NA, c(I,J,K,NRI))
     count <- -1
     for (k in 1:K) {
       for (j in 1:J) {
         for (i in 1:I) {
-          if (img.mask[i,j,k]) {
+          if (mask[i,j,k]) {
             count <- count + 1
             A[i,j,k,] <- sample[(1:NRI) + count*NRI]
           }
         }
       }
     }
-    return(A)
+    return(drop(A))
   }
 
-  if (nriters < thin)
+  extractSamples <- function(sample, img.mask, NRI) {
+    out <- array(NA, c(NRI, dim(img.mask)))
+    out[img.mask] <- sample
+    aperm(out, c(2:length(dim(out)), 1)) # not too sure about drop()
+  }
+
+  I <- nrow(conc)
+  J <- ncol(conc)
+  K <- nsli(conc)
+  if (!is.numeric(dim(conc))) {
+    I <- J <- K <- 1
+  } else {
+    if (length(dim(conc)) == 2) {
+      J <- K <- 1
+    }
+    if (length(dim(conc)) == 3) {
+      K <- 1
+     }
+  }
+
+  if (J>1&K>1)
+  if (sum(dim(img.mask) - dim(conc)[-length(dim(conc))]) != 0) {
+    stop("Dimensions of \"conc\" do not agree with \"img.mask\"")
+  }
+  if (nriters < thin) {
     stop("Please check settings for nriters")
-  if (burnin < 0)
+  }
+  if (burnin < 0) {
     stop("Please check settings for burnin")
-  if (thin < 1)
+  }
+  if (thin < 1) {
     stop("Please check settings for thin")
-  if (tune < 50)
+  }
+  if (tune < 50) {
     stop("Please check settings for tune")
+  }
 
   if (burnin < tune) {
     burnin <- tune
@@ -151,6 +183,7 @@ setMethod("dcemri.bayes", signature(conc="array"),
                            stop("Only aif=\"tofts.kermode\" or aif=\"fritz.hansen\" acceptable AIFs for model=\"weinmann\" or model=\"extended\"", call.=FALSE))
                   }
                 },
+                kety.orton.exp = ,
                 orton.exp = {
                   if (is.null(aif)) {
                     "orton.exp"
@@ -165,36 +198,30 @@ setMethod("dcemri.bayes", signature(conc="array"),
 
   mod <- model
   nvoxels <- sum(img.mask)
-  I <- nrow(conc)
-  J <- ncol(conc)
-  K <- nsli(conc)
-  
-  if (!is.numeric(dim(conc))) {
-    I <- J <- K <- 1
-  } else {
-    if (length(dim(conc)) == 2) {
-      J <- K <- 1
-    }
-  }
-
-  img.mask <- array(img.mask,c(I,J,K))
- 
-  if (verbose) cat("  Deconstructing data...", fill=TRUE)
-  img.mask <- ifelse(img.mask > 0, TRUE, FALSE)
-  conc.mat <- matrix(conc[img.mask], nvoxels)
-  conc.mat[is.na(conc.mat)] <- 0
 
   switch(aif,
          tofts.kermode = {
-           D <- 0.1; a1 <- 3.99; a2 <- 4.78; m1 <- 0.144; m2 <- 0.0111
+           D <- 0.1
+           a1 <- 3.99
+           a2 <- 4.78
+           m1 <- 0.144
+           m2 <- 0.0111
            aif.parameter <- c(D*a1, m1, D*a2, m2)
          },
          fritz.hansen = {
-           D <- 1; a1 <- 2.4; a2 <- 0.62; m1 <- 3.0; m2 <- 0.016
+           D <- 1
+           a1 <- 2.4
+           a2 <- 0.62
+           m1 <- 3.0
+           m2 <- 0.016
            aif.parameter <- c(D*a1, m1, D*a2, m2)
          },
          orton.exp = {
-           D <- 1; a1 <- 323; m1 <- 20.2; a2 <- 1.07; m2 <- 0.172
+           D <- 1
+           a1 <- 323
+           m1 <- 20.2
+           a2 <- 1.07
+           m2 <- 0.172
            aif.parameter <- c(D*a1, m1, D*a2, m2)
          },
          ## FIXME orton.cos does not seem to be implemented
@@ -203,9 +230,13 @@ setMethod("dcemri.bayes", signature(conc="array"),
          ##  aif.parameter=c(D*a1, m1, D*a2, m2)
          ##},
          user = {
-           if (verbose) cat("  User-specified AIF parameters...", fill=TRUE);
-           D <- try(user$D); AB <- try(user$AB) 
-           muB <- try(user$muB); AG <- try(user$AG)
+           if (verbose) {
+             cat("  User-specified AIF parameters...", fill=TRUE)
+           }
+           D <- try(user$D)
+           AB <- try(user$AB) 
+           muB <- try(user$muB)
+           AG <- try(user$AG)
            muG <- try(user$muG)
            aif.parameter <- c(D * AB, muB, D * AG, muG)
            ## aG, aB are probably related to orton.cos which isn't implemented
@@ -218,98 +249,174 @@ setMethod("dcemri.bayes", signature(conc="array"),
          weinmann = { aif.model <- 0 ; vp.do <- 0 },
          extended = { aif.model <- 0 ; vp.do <- 1 },
          orton.exp = { aif.model <- 1 ; vp.do <- 1 },
+         kety.orton.exp = { aif.model <- 1 ; vp.do <- 0 },
          stop("Model is not supported."))
 
+  img.mask <- array(img.mask,c(I,J,K)) # why?
+ 
+  if (verbose) {
+    cat("  Deconstructing data...", fill=TRUE)
+  }
+  img.mask <- ifelse(img.mask > 0, TRUE, FALSE)
+  conc.mat <- matrix(conc[img.mask], nvoxels)
+  conc.mat[is.na(conc.mat)] <- 0
+
+  conc.list <- vector("list", nvoxels) # list()
+  for (i in 1:nvoxels) {
+    conc.list[[i]] <- conc.mat[i,]
+  }
+
+  if (verbose) {
+    cat("  Estimating the kinetic parameters...", fill=TRUE)
+  }
+
+  if (multicore && require("multicore")) {
+    bayes.list <- mclapply(conc.list, FUN=.dcemri.bayes.single,
+                           time=time, nriters=nriters, thin=thin,
+                           burnin=burnin, tune=tune, ab.gamma=ab.ktrans,
+                           ab.theta=ab.kep, ab.vp=ab.vp,
+                           ab.tauepsilon=ab.tauepsilon, aif.model=aif.model,
+                           aif.parameter=aif.parameter, vp=vp.do)
+  } else {
+    bayes.list <- lapply(conc.list, FUN=.dcemri.bayes.single, time=time,
+                         nriters=nriters, thin=thin, burnin=burnin,
+                         tune=tune, ab.gamma=ab.ktrans, ab.theta=ab.kep,
+                         ab.vp=ab.vp, ab.tauepsilon=ab.tauepsilon,
+                         aif.model=aif.model, aif.parameter=aif.parameter,
+                         vp=vp.do)    
+  }
+  rm(conc.list) ; gc()
+
+  if (verbose) {
+    cat("  Extracting results...", fill=TRUE)
+  }
+
+  n <- (nriters - tune) / thin # number of samples from posterior
   ktrans <- kep <- list(par=rep(NA, nvoxels), error=rep(NA, nvoxels))
   sigma2 <- rep(NA, nvoxels)
-  Vp <- list(par=rep(NA, nvoxels), error=rep(NA, nvoxels))
+  if (mod %in% c("extended", "orton.exp", "orton.cos")) {
+    Vp <- list(par=rep(NA, nvoxels), error=rep(NA, nvoxels))
+  }
+  if (dic) {
+    med.deviance <- rep(NA, nvoxels)
+  }
   if (samples) {
-    sigma2.samples <- ktrans.samples <- kep.samples <- c()
-    if (mod %in% c("extended", "orton.exp", "orton.cos"))
-      Vp.samples <- c()
+    sigma2.samples <- ktrans.samples <- kep.samples <- rep(NA, n*nvoxels) # c() # NULL # 
+    if (mod %in% c("extended", "orton.exp", "orton.cos")) {
+      Vp.samples <- rep(NA, n*nvoxels) # c() # NULL #
+    }
+    if (dic) {
+      deviance.samples <- rep(NA, n*nvoxels)
+    }
   }
 
-  if (verbose) cat("  Estimating the kinetic parameters...", fill=TRUE)
-
-  conc.list <- list()
-  for (i in 1:nvoxels)
-    conc.list[[i]] <- conc.mat[i,]
-
-  if (!multicore) {
-    fit <- lapply(conc.list, FUN=.dcemri.bayes.single,
-                  time=time, nriters=nriters, thin=thin, burnin=burnin,
-                  tune=tune, ab.vp=ab.vp, ab.tauepsilon=ab.tauepsilon,
-                  aif.model=aif.model, aif.parameter=aif.parameter,
-		  vp=vp.do)
-  } else {
-    require("multicore")
-    fit <- mclapply(conc.list, FUN=.dcemri.bayes.single,
-                  time=time, nriters=nriters, thin=thin, burnin=burnin,
-                  tune=tune, ab.vp=ab.vp, ab.tauepsilon=ab.tauepsilon,
-                  aif.model=aif.model, aif.parameter=aif.parameter, 
-                  vp=vp.do)
-  }
-
-  if (verbose) cat("  Reconstructing results...", fill=TRUE)
-
   for (k in 1:nvoxels) {
-    ktrans$par[k] <- median(fit[[k]]$ktrans)
-    kep$par[k] <- median(fit[[k]]$kep)
-    ktrans$error[k] <- sqrt(var(fit[[k]]$ktrans))
-    kep$error[k] <- sqrt(var(fit[[k]]$kep))
+    ##if (verbose && trunc(k/100) == k/100) {
+    ##  cat("  - k =", k, fill=TRUE)
+    ##}
+    index <- ((k-1)*n):(k*n)
+    ktrans$par[k] <- median(bayes.list[[k]]$ktrans)
+    kep$par[k] <- median(bayes.list[[k]]$kep)
+    ktrans$error[k] <- sd(bayes.list[[k]]$ktrans)
+    kep$error[k] <- sd(bayes.list[[k]]$kep)
     if (mod %in% c("extended", "orton.exp", "orton.cos")) {
-      Vp$par[k] <- median(fit[[k]]$vp)
-      Vp$error[k] <- sqrt(var(fit[[k]]$vp))
-      if (samples)
-	Vp.samples <- c(Vp.samples,fit[[k]]$v)
+      Vp$par[k] <- median(bayes.list[[k]]$vp)
+      Vp$error[k] <- sd(bayes.list[[k]]$vp)
+      if (samples) {
+	##Vp.samples <- c(Vp.samples, bayes.list[[k]]$v)
+        Vp.samples[index] <- bayes.list[[k]]$v
+      }
+    }
+    sigma2[k] <- median(bayes.list[[k]]$sigma2)
+    if (dic) {
+      med.deviance[k] <- median(bayes.list[[k]]$deviance)
     }
-    sigma2[k] <- median(fit[[k]]$sigma2)
     if (samples) {
-      ktrans.samples <- c(ktrans.samples,fit[[k]]$ktrans)
-      kep.samples <- c(kep.samples,fit[[k]]$kep)
-      sigma2.samples <- c(sigma2.samples,fit[[k]]$sigma2)
+      ##ktrans.samples <-  c(ktrans.samples, bayes.list[[k]]$ktrans)
+      ktrans.samples[index] <- bayes.list[[k]]$ktrans
+      ##kep.samples <- c(kep.samples, bayes.list[[k]]$kep)
+      kep.samples[index] <- bayes.list[[k]]$kep
+      ##sigma2.samples <- c(sigma2.samples, bayes.list[[k]]$sigma2)
+      sigma2.samples[index] <- bayes.list[[k]]$sigma2
+      if (dic) {
+        ##deviance.samples <- c(deviance.samples, bayes.list[[k]]$deviance)
+        deviance.samples[index] <- bayes.list[[k]]$deviance
+      }
     }
   }
+  rm(bayes.list) ; gc()
+
+  if (verbose) {
+    cat("  Reconstructing results...", fill=TRUE)
+  }
 
   A <- B <- array(NA, c(I,J,K))
   A[img.mask] <- ktrans$par
   B[img.mask] <- ktrans$error
   ktrans.out <- list(par=A, error=B)
+  rm(A,B,ktrans)
   A <- B <- array(NA, c(I,J,K))
   A[img.mask] <- kep$par
   B[img.mask] <- kep$error
   kep.out <- list(par=A, error=B)
-
+  rm(A,B,kep)
   if (mod %in% c("extended", "orton.exp", "orton.cos")) {
     A <- B <- array(NA, c(I,J,K))
     A[img.mask] <- Vp$par 
     B[img.mask] <- Vp$error
     Vp.out <- list(par=A, error=B)
+    rm(A,B,Vp)
   }
 
-  A <- B <- array(NA, c(I,J,K))
+  A <- array(NA, c(I,J,K))
   A[img.mask] <- sigma2
   sigma2.out <- A
+  rm(A,sigma2)
 
-  if (samples) {
-    NRI <- length(ktrans.samples) / length(ktrans$par)
-    ktrans.out <- list(par=ktrans.out$par,
-                       error=ktrans.out$error,
-                       samples=extract.samples(ktrans.samples,I,J,K,NRI))
-    kep.out <- list(par=kep.out$par,
-                    error=kep.out$error,
-                    samples=extract.samples(kep.samples,I,J,K,NRI))
-    if (mod %in% c("extended", "orton.exp", "orton.cos"))
-      Vp.out <- list(par=Vp.out$par,
-                     error=Vp.out$error,
-                     samples=extract.samples(Vp.samples, I, J, K, NRI))
-    sigma2.samples <- extract.samples(sigma2.samples, I, J, K, NRI)
+  if (dic) {
+    A <- array(NA, c(I,J,K))
+    A[img.mask] <- med.deviance
+    med.deviance <- A
+    rm(A)
   }
 
-  returnable <- list(ktrans=ktrans.out$par, kep=kep.out$par,
-                     ktranserror=ktrans.out$error, keperror=kep.out$error, 
-                     ve=ktrans.out$par/kep.out$par, sigma2=sigma2.out,
+  if (samples) {
+    if (verbose) {
+      cat("  Reconstructing samples...", fill=TRUE)
+    }
+#    NRI <- length(ktrans.samples) / length(ktrans$par)
+#    ktrans.out <- list(par=ktrans.out$par,
+#                       error=ktrans.out$error,
+#                       samples=extract.samples(ktrans.samples,img.mask,I,J,K,NRI))
+    ktrans.out$samples <- extractSamples(ktrans.samples, img.mask, n)
+#    kep.out <- list(par=kep.out$par,
+#                    error=kep.out$error,
+#                    samples=extract.samples(kep.samples,img.mask,I,J,K,NRI))
+    kep.out$samples <- extractSamples(kep.samples, img.mask, n)
+    if (mod %in% c("extended", "orton.exp", "orton.cos")) {
+#      Vp.out <- list(par=Vp.out$par,
+#                     error=Vp.out$error,
+#                     samples=extract.samples(Vp.samples, img.mask, I, J, K, NRI))
+      Vp.out$samples <- extractSamples(Vp.samples, img.mask, n)
+    }
+    sigma2.samples <- extractSamples(sigma2.samples, img.mask, n)
+    if (dic) {
+      deviance.samples <- extractSamples(deviance.samples, img.mask, n)
+    }
+  }
+
+  returnable <- list(ktrans=ktrans.out$par,
+                     kep=kep.out$par,
+                     ktranserror=ktrans.out$error,
+                     keperror=kep.out$error,
+                     ve=ktrans.out$par/kep.out$par,
                      time=time)
+#  returnable[["kep"]] <- kep.out$par
+#  returnable[["ktranserror"]] <- ktrans.out$error
+#  returnable[["keperror"]] <- kep.out$error 
+#  returnable[["ve"]] <- ktrans.out$par/kep.out$par
+#  returnable[["sigma2"]] <- sigma2.out
+#  returnable[["time"]] <- time
 
   if (mod %in% c("extended", "orton.exp", "orton.cos")) {
     returnable[["vp"]] <- Vp.out$par
@@ -318,11 +425,64 @@ setMethod("dcemri.bayes", signature(conc="array"),
       returnable[["vp.samples"]] <- Vp.out$samples
     } 
   } 
+
   if (samples) {
-    returnable[["ktrans.samples"]] <- ktrans.out$samples
-    returnable[["kep.samples"]] <- kep.out$samples
+    temp <- ktrans.out$samples
+    rm(ktrans.out)
+    returnable[["ktrans.samples"]] <- temp
+    temp <- kep.out$samples
+    rm(kep.out)
+    returnable[["kep.samples"]] <- temp
     returnable[["sigma2.samples"]] <- sigma2.samples
   }
+
+  ## DIC
+
+  if (dic) {
+    if (verbose) {
+      cat("  Computing DIC...", fill=TRUE)
+    }
+    fitted <- array(NA,c(I,J,K,length(time)))
+    for (i in 1:I) {
+      for (j in 1:J) {
+        for (k in 1:K) {
+          if (img.mask[i,j,k]) {
+            par <- list("ktrans"=ktrans.out$par[i,j,k],
+                        "kep"=kep.out$par[i,j,k])
+            if (vp.do) {
+              par["vp"] <- Vp.out$par[i,j,k]
+            }
+            fitted[i,j,k,] <- kineticModel(time,par,model=model,aif=aif)
+          }
+        }
+      }
+    }
+    fitted <- fitted - conc
+    fitted <- fitted * fitted
+    fitted <- apply(fitted, 1:3, sum)
+
+    deviance.med <- length(time) * log(sigma2.out) + fitted / sigma2.out
+    pD <- med.deviance - deviance.med
+    DIC <- med.deviance + pD
+    returnable[["DIC"]] <- sum(DIC,na.rm=TRUE)
+    returnable[["pD"]] <- sum(pD,na.rm=TRUE)
+    returnable[["DIC.map"]] <- DIC
+    returnable[["pD.map"]] <- pD 
+    returnable[["deviance.med"]] <- deviance.med
+    returnable[["med.deviance"]] <- med.deviance
+    if (samples) {
+      returnable[["deviance.samples"]] <- deviance.samples
+    }
+    rm(DIC)
+    rm(pD)
+    rm(deviance.med)
+    rm(med.deviance)
+    rm(deviance.samples)
+    gc()
+  }
+
+  rm(Vp.out) ; gc()
+
   return(returnable)
 }
 
